from .baseengine import BaseEngine
import pymorphy2


class Pymorphy2Engine(BaseEngine):
    """docstring for PyMorphy2"""
    def parse(self):
        self.morph = pymorphy2.MorphAnalyzer()
        self.parsed_word = self.morph.parse(self.word)[0]

    def normal_form(self):
        self.normal_form_word = self.parsed_word.normal_form

    def parse_normal(self):
        self.parsed_normal_word = self.morph.parse(self.normal_form_word)[0]

    def lexeme(self):
        lexemes = self.parsed_normal_word.lexeme
        self.lexemes = [lexeme.word for lexeme in lexemes if len(lexeme.word)>2]




# -*- coding: utf-8 -*- 
from main import EpubWorker
import unittest
# import subprocess

class TestMainPy(unittest.TestCase):
    def setUp(self):
        pass


WRONG_REQUEST = {'results': []}
RIGHT_REQUEST = {'baseCfi': '/6/24[id15]!', 'title': '', 'path': './tmp/Sensei4//index_split_011.html',
                              'href': 'index_split_011.html', 'cfi': '/6/24[id15]!/4/62'}
CHARACTER_CFI = ['/6/2[id116]', '/6/4[id115]', '/6/6[id114]', '/6/8[id113]', '/6/10[id112]', '/6/12[id111]', '/6/14[id110]',
 '/6/16[id19]', '/6/18[id18]', '/6/20[id17]', '/6/22[id16]', '/6/24[id15]', '/6/26[id14]', '/6/28[id13]', '/6/30[id12]', '/6/32[id11]']


class TestEpubWorker(unittest.TestCase):
    def setUp(self):
        test_dir = './test_data/'
        self.epub_dir = EpubWorker(test_dir + 'Sensei4/')
        self.epub_file = EpubWorker(test_dir + 'Sensei4.epub')

    def test_search_in_dir(self):
        self.assertDictEqual(self.epub_dir.search_word('хряк'), WRONG_REQUEST)
        self.assertDictEqual(self.epub_dir.search_word('аллат')['results'][0], RIGHT_REQUEST)

    def test_search_in_epub(self):
        self.assertDictEqual(self.epub_file.search_word('хряк'), WRONG_REQUEST)
        self.assertDictEqual(self.epub_file.search_word('аллат')['results'][0], RIGHT_REQUEST)

    def test_force_index(self):
        pass

    def test_get_old_book_from_index(self):
        pass

    def test_get_character_cfi(self):
        self.assertEqual(self.epub_file.get_characters_cfi(), CHARACTER_CFI)
        self.assertEqual(self.epub_dir.get_characters_cfi(), CHARACTER_CFI)


if __name__ == '__main__':
    unittest.main()




from flask import Flask
from flask import request
from flask import jsonify
from flask import g

from epubsearch import EpubIndexer
from epubsearch import EpubParser
from epubsearch import crossdomain

app = Flask(__name__)
epub = EpubParser("moby-dick")

index = EpubIndexer("whoosh")
index.load(epub)

@app.route("/")
def home():
    return "try /search?q=whale"

@app.route("/search", methods=['GET','OPTIONS'])
@crossdomain(origin='*')
def search():
    query = request.args.get('q')
    results = index.search(query)
    print(results)
    return jsonify(**results)

if __name__ == "__main__":
    app.run(debug=True)




import importlib
from lxml import etree
import re, os

import logging


class EpubIndexer(object):
    epub = False
    engine = False

    def __init__(self, engine_name=False, database_name='indexdir', force_index=False):
        self.force_index = force_index
        self.database_name = database_name

        database_folder_path = "/tmp/epub_worker/databases/"
        if not os.path.exists(database_folder_path):
            os.mkdir(database_folder_path)

        self.database_path = database_folder_path + database_name
        if engine_name:
            mod = importlib.import_module("epubsearcher.epubsearch.search_engines.%sengine" % engine_name)
            # import whooshengine as engine
            self.engine = getattr(mod,'%sEngine' % engine_name.capitalize())(database_name)
            logging.info(self.engine)

    def load(self, epub):
        self.epub = epub
        if os.path.exists(self.database_path) and not self.force_index:
            try:
                self.engine.open()
            except Exception as e:
                print(e)
        else:
            self.engine.create()

            for spineItem in epub.spine:

                path = epub.base + "/" + spineItem['href']

                self.engine.add(path=path, href=spineItem['href'], title=spineItem['title'], cfiBase=spineItem['cfiBase'], spinePos=spineItem['spinePos'])

            self.engine.finished()

    def search(self, q, limit=None):
        rawresults = self.engine.query(q, limit)
        # print len(rawresults)
        r = {}
        r["results"] = []
        q = q.lower()

        for hit in rawresults:
            baseitem = {}
            baseitem['title'] = hit["title"].decode(encoding="UTF-8")
            baseitem['href'] = hit["href"].decode(encoding="UTF-8")
            baseitem['path'] = hit["path"].decode(encoding="UTF-8")

            # find base of cfi
            cfi_base= hit['cfiBase'].decode(encoding="UTF-8") + "!"

            with open(hit["path"]) as fileobj:
                tree = etree.parse(fileobj)
                parsedString = etree.tostring(tree.getroot())
                # case-insensitive xpath search
                xpath = './/*[contains(translate(text(), "ABCDEFGHIJKLMNOPQRSTUVWXYZ", "abcdefghijklmnopqrstuvwxyz") , "'+ q + '")]'
                #xpath = './/*[contains(text(),"'+ q +'")]'

                matchedList = tree.xpath(xpath)
                # print len(matchedList)
                for word in matchedList:
                    # copy the base
                    item = baseitem.copy()

                    # print word
                    # print word.getparent()
                    item['baseCfi'] = cfi_base
                    item['cfi'] = get_cfi(cfi_base, word)
                    #print cfi
                    r["results"].append(item)

        ## Sort results by chapter
        r['results'] = sorted(r['results'], key=lambda x: get_cfi_chapter(x['baseCfi']))
        return r


def get_cfi(cfiBase, word):

    cfi_list = []
    parent = word.getparent()
    child = word
    while parent is not None:
        i = parent.index(child)
        if 'id' in child.attrib:
            cfi_list.insert(0,str((i+1)*2)+'[' + child.attrib['id'] + ']')
        else:
            cfi_list.insert(0,str((i+1)*2))
        child = parent
        parent = child.getparent()

    cfi = cfiBase + '/' + '/'.join(cfi_list)
    return cfi

def get_cfi_chapter(cfi_base):
    cfi_base = re.sub(r'\[.*\]','',cfi_base)
    chapter_location = cfi_base[cfi_base.rfind('/')+1:cfi_base.find('!')]
    return int(chapter_location)



import xml.etree.ElementTree as ET
from lxml import etree

import logging
import os

class EpubParser(object):
    base = ''
    manifest = {}
    titles = {}
    spine = []
    spine_element_num = str(6) #replace this with xml check for which child of root 'spine' is x 2

    def __init__(self, path):
        self.name = path.replace('/', '')
        
        folder = path  + "/"
        if os.path.isdir(folder) == True:
            rootfile = self.parse_root_file(folder)
            if rootfile:
                self.rootfile = rootfile
                self.base = folder + os.path.dirname(rootfile)
                self.manifest = self.parse_manifest(folder + rootfile)
                self.tocPath = self.get_toc_path(self.manifest)

            if self.tocPath:
                self.toc = self.parse_toc(self.base + "/" + self.tocPath)

            self.spine = self.parse_spine(folder + rootfile)
        else :
            raise EpubError("No Rootfile found")

    def parse_root_file(self, folder):
        container = folder + "/META-INF/container.xml"

        if os.path.isfile(container):
            tree = ET.parse(container)
            root = tree.getroot()

            for rootfiles in root:
                for rf in rootfiles:
                    rootfile = rf.attrib['full-path']
                    return rootfile # Stop at first rootfile for now

        else:
            raise EpubError('No container.xml found')

        return False

    def parse_metadata(self, filename):
        raise NotImplementedError

    def parse_manifest(self, filename):
        """
        Parse the content.opf file.
        """
        namespaces = {'xmlns': 'http://www.idpf.org/2007/opf',
            'dc':'http://purl.org/dc/elements/1.1/',
            'dcterms':'http://purl.org/dc/terms/'}

        logging.debug("Parsing Manifest")

        items = {}

        # begin parsing content.opf
        tree = ET.parse(filename)
        root = tree.getroot()
        # extract item hrefs and place in return list
        for child in root.findall('xmlns:manifest/xmlns:item', namespaces=namespaces):
            item_id = child.attrib['id']

            if not item_id in items:
                items[item_id] = {}
                items[item_id]["href"] = child.attrib['href']
                items[item_id]["media-type"] = child.attrib['media-type']

        return items

    def get_toc_path(self, manifest):
        """
        From the manifest items, find the item for the Toc
        Return the Toc href
        """
        if "ncxtoc" in manifest:
            return manifest["ncxtoc"]["href"]
        else:
            for item in manifest.values():
                if item["media-type"] == "application/x-dtbncx+xml":
                    return item["href"]

        raise EpubError("No Toc File Found")

    def parse_toc(self, filename):
        namespaces = {'xmlns': 'http://www.daisy.org/z3986/2005/ncx/'}

        logging.debug("Parsing TOC")

        items = []

        # begin parsing content.opf
        #tree = ET.parse(filename)
        #root = tree.getroot()
        tree = etree.parse(filename)
        root = tree.getroot()

        # extract item hrefs and place in return list
        for nav_point in root.findall('xmlns:navMap/xmlns:navPoint', namespaces=namespaces):
            nav_label = nav_point.find('xmlns:navLabel', namespaces=namespaces)
            title = nav_label.getchildren()[0].text.encode("utf-8")
            href = nav_point.getchildren()[1].attrib['src'].encode("utf-8")
            item = {}

            item['id'] = nav_point.attrib['id']
            item['title'] = title
            self.titles[href] = title
            items.append(item)

        return items

    def parse_spine(self, filename):
        """
            Parse the content.opf file.
        """
        namespaces = {'xmlns': 'http://www.idpf.org/2007/opf',
                'dc':'http://purl.org/dc/elements/1.1/',
                'dcterms':'http://purl.org/dc/terms/'}

        logging.debug("Parsing Spine")

        spinePos = 1

        items = []

        # begin parsing content.opf
        tree = ET.parse(filename)
        root = tree.getroot()
        # extract item hrefs and place in return list
        for child in root.findall('xmlns:spine/xmlns:itemref', namespaces=namespaces):
            item = {}

            item["idref"] = child.attrib['idref']
            item["spinePos"] = spinePos

            item["cfiBase"] = "/" + "/".join([ self.spine_element_num, str(spinePos*2) + '[{}]'.format(item['idref']) ])

            if item["idref"] in self.manifest:
                item["href"] = self.manifest[item["idref"]]['href']

            # print self.titles
            if self.titles and item["href"] in self.titles:
                item['title'] = self.titles[item["href"]]
            else:
                item['title'] = ''

            items.append(item)

            spinePos += 1

        return items

class EpubError(Exception):
    pass



from whoosh.index import create_in
import whoosh.index as index
from whoosh.fields import Schema, TEXT, ID
from bs4 import BeautifulSoup
from whoosh.qparser import QueryParser
from .baseengine import BaseEngine
import os
import re
import logging


class WhooshEngine(BaseEngine):
    # whoosh
    schema = Schema(title=TEXT(stored=True), path=TEXT(stored=True), href=ID(stored=True), cfiBase=TEXT(stored=True), spinePos=TEXT(stored=True), content=TEXT)

    def open(self):
        try:
            self.ix = index.open_dir(self.database_path)
        except Exception as e:
            logging.error("openning database {} failed".format(self.database_name))

    def create(self):

        if not os.path.exists(self.database_path):
            os.mkdir(self.database_path)

        try:
            logging.debug("openning database {} to create".format(self.database_name))
            self.ix = index.create_in(self.database_path, self.schema)
        except Exception as e:
            logging.error(e)

        self.writer = self.ix.writer()

    def add(self, path='', href='', title='', cfiBase='', spinePos=''):
        text = self.__get_text(path)
        self.writer.add_document(title=str(title), path=str(path), href=str(href), cfiBase=str(cfiBase), spinePos=str(spinePos), content=str(text))
        logging.debug("Indexed: " + title + ' | ' + path + ' | ' + href + ' | ' + str(spinePos))

    def finished(self):
        self.writer.commit()

    def query(self, q, limit=None):
        logging.debug('Q {}'.format(q))
        with self.ix.searcher() as searcher:
            results = []
            parsed_query = QueryParser("content", schema=self.ix.schema).parse(q)
            hits = searcher.search(parsed_query, limit=limit)
            logging.debug("Hits {}".format(hits))
            for hit in hits:
                item = {}
                item['title'] = hit["title"].encode("utf-8")
                item['href'] = hit["href"].encode("utf-8")
                item['path'] = hit["path"].encode("utf-8")
                item['cfiBase'] = hit["cfiBase"].encode("utf-8")
                item['spinePos'] = hit["spinePos"].encode("utf-8")
                results.append(item)

            return results

    def __get_text(self, filename):
        # html = urllib.urlopen('http://www.nytimes.com/2009/12/21/us/21storm.html').read()
        html = open(filename, "r")
        soup = BeautifulSoup(html)
        texts = soup.findAll(text=True)

        def visible(element):
                if element.parent.name in ['style', 'script', '[document]', 'head', 'title']:
                        return False
                elif re.match('<!--.*-->', str(element.encode('utf-8'))):
                        return False
                return True

        visible_texts = filter(visible, texts)

        contents = ' '.join([s for s in visible_texts])

        return contents.strip() #.encode('utf-8')



import logging
import os
import re
import pdb
from .baseengine import BaseEngine
from cheshire3.baseObjects import Session
from cheshire3.server import SimpleServer
from cheshire3.internal import cheshire3Root
from cheshire3.exceptions import ObjectDoesNotExistException
import json


class Cheshire3Engine(BaseEngine):
    #schema = Schema(title=TEXT(stored=True), path=TEXT(stored=True), href=ID(stored=True), cfiBase=TEXT(stored=True), spinePos=TEXT(stored=True), content=TEXT)
    #database = 'db_tdo_simple_sru'
    cheshire_metadata_dir = '/cheshire3-metadata'
    session = Session()
    serverConfig = os.path.join(cheshire3Root, 'configs', 'serverConfig.xml')
    server = SimpleServer(session, serverConfig)
    queryFactory = None
    db = None
    titleSel = None
    anywhereSel = None
    proxExtractor = None

    def __initializeTitleSelector(self):
        try:
            self.titleSel = self.db.get_object(self.session, 'titleXPathSelector')
        except ObjectDoesNotExistException:
            try:
                self.titleSel = self.db.get_object(self.session, 'titleSelector')
            except ObjectDoesNotExistException as e:
                logging.error(e)

    def __initializeAnywhereSelector(self):
        try:
            self.anywhereSel = self.db.get_object(self.session, 'anywhereXPathSelector')
        except ObjectDoesNotExistException as e:
            logging.error(e)

    def __initializeProximityExtractor(self):
        try:
            self.proxExtractor = self.db.get_object(self.session, 'ProxExtractor')
        except ObjectDoesNotExistException as e:
            logging.error(e)

    def __highlight(self, text, term, n):
        """Searches for text, retrieves n words either side of the text, which are retuned seperately"""
        term_concordance = list()
        text_len = len(text)
        term_len = len(term)
        term_indexes = [w.start() for w in re.finditer(term, text)]
        for idx in term_indexes:
            start = idx - n
            end = text_len if (idx + term_len + n) > text_len else idx + term_len + n
            term_concordance.append(text[start:idx] + '<b class="match term0">' + term + '</b>' + text[idx:end]) 

        return term_concordance

    def open(self):
        """ The Cheshire get_object line should throw an exception if it can't 
        open passed db
        """
        try:
            self.db = self.server.get_object(self.session, self.database_name)
            self.session.database = self.database_name
        except Exception as e:
            logging.error(e)
            logging.error( "openning database {} failed".format(self.database_name))

    def create(self):
        if not os.path.exists(self.database_path):
            os.makedirs(self.database_path)

        # create cheshire metadata directory if needed, then initialize with empty list
        metadata_path = self.database_path + self.cheshire_metadata_dir
        if not os.path.exists(metadata_path):
            os.makedirs(metadata_path)
        with open(metadata_path + '/' + self.database_name, 'w') as f:
            json.dump({}, f)

        try:
            logging.info("openning database {} to create".format(self.database_path))
            os.system("cheshire3-init " + self.database_path + " --database=" + self.database_name)
        except Exception as e:
            logging.error(e)

    def add(self, path='', href='', title='', cfiBase='', spinePos=''):
        # first, index the document in cheshire3 using unix commands
        os.system("cheshire3-load --database=" + self.database_name + ' ' + path)
        
        doc_md = dict()
        doc_md[href] = {'path' : path, 'href' : href, 'title' : title, 'cfiBase' : cfiBase, 'spinePos' : spinePos}
        # title is not populated, so pulling filename from path prefix
        #filename = path[:path.find('/')] + '.json'
        metadata_path = self.database_path + self.cheshire_metadata_dir
        with open(metadata_path + '/' + self.database_name) as f_in:
            md_dict = json.load(f_in)
            
        md_dict.update(doc_md)

        with open(metadata_path + '/' + self.database_name, 'w') as f_out:
            json.dump(md_dict, f_out)
        #print "Current Path for directory writing: " + os.getcwd()

    def finished(self):
        """ In Cheshire, there are no cleanup commands that are needed.  The add command
            will index specified documents fully and end, so a finished command is not required.
        """
        pass

    def query(self, q, limit=None):
        """ In Cheshire3, you have to specify an index and query, else it defaults the all index  which utilizes simple extraction.
        """

        if self.queryFactory == None:
            self.queryFactory = self.db.get_object(self.session, 'defaultQueryFactory')

        if self.titleSel is None:
            self.__initializeTitleSelector()

        if self.anywhereSel is None:
            self.__initializeAnywhereSelector()

        if self.proxExtractor is None:
            self.__initializeProximityExtractor()

        c3Query = self.queryFactory.get_query(self.session, q)
        rs = self.db.search(self.session, c3Query)

        # open up the json file with reader specific attributes
        metadata_path = self.database_path + self.cheshire_metadata_dir
        with open(metadata_path + '/' + self.database_name) as f:
            db_md_dict = json.load(f)

        # loop through recordset, create new results list with dictionary of found values
        results = list()
        for rsi in rs:
            rec = rsi.fetch_record(self.session)
            # check the record titles
            titleData = self.titleSel.process_record(self.session, rec)
            # checking out the proximity attributes
            elems = self.anywhereSel.process_record(self.session, rec)           
            doc_dict = self.proxExtractor.process_xpathResult(self.session, elems).values()[0]
            concordance = self.__highlight(doc_dict['text'], q, 20)
            pdb.set_trace()
            # extracts document name key
            fn_key = os.path.basename(titleData[3][0])
            # append highlighted concordance to the dictionary
            db_md_dict[fn_key][u'highlight'] = "  ".join(concordance)
            results.append(db_md_dict[fn_key])
        return results



